import json, os, tarfile

from pip._vendor.packaging.version import Version
from pip.index import FormatControl, Link
from pip.req import InstallRequirement
from pip.download import PipSession

from piptools.repositories.base import BaseRepository
from piptools.utils import as_tuple, make_install_requirement

import pytest


def pytest_addoption(parser):
    parser.addoption('--run-pypi-integration-tests', action='store_true')


@pytest.fixture
def FakePypiRepository(tmpdir):
    FakeRepository.tmpdir = str(tmpdir)
    return FakeRepository


class FakePackageFinder:
    format_control = FormatControl(set(), set())
    def find_requirement(self, ireq, upgrade):
        return Link(str(ireq)) # this URL will be passed to FakePipSession.get - we ignore extras for now
    def add_dependency_links(self, links):
        pass


class FakeRepository(BaseRepository):
    """
    Mock for pip_compile.PyPIRepository
    Inspired by: https://raw.githubusercontent.com/nvie/pip-tools/1.6.5/tests/conftest.py
    """
    finder = FakePackageFinder()
    tmpdir = None

    @property
    def source_dir(self):
        return self.tmpdir

    @property
    def _download_dir(self):
        return self.tmpdir

    def __init__(self, *args, **kwargs):
        with open('tests/fixtures/fake-index.json', 'r') as f:
            self.index = json.load(f)
        self.session = FakePipSession(self.tmpdir, self.index)

    def find_best_match(self, ireq, prereleases=False):
        if ireq.editable:
            return ireq
        versions = ireq.specifier.filter(self.index[ireq.req.key], prereleases=prereleases)
        best_version = max(versions, key=Version)
        return make_install_requirement(ireq.req.key, best_version, ireq.extras)

    def get_dependencies(self, ireq):
        name, version, extras = as_tuple(ireq)
        # Store non-extra dependencies under the empty string
        extras = ireq.extras + ('',)
        dependencies = [dep for extra in extras for dep in self.index[name][version][extra]]
        return [InstallRequirement.from_line(dep) for dep in dependencies]

    def get_hashes(self, ireq):
        raise NotImplementedError # not needed

class FakePipSession(PipSession):
    def __init__(self, tmpdir, index):
        self.tmpdir = tmpdir
        self.index = index

    def get(self, url, headers, stream):
        'Returns a mock valid tarball containing a single setup.py'
        assert '==' in url, '"url" should come from FakePackageFinder.find_requirement'
        pkg_name, version = url.split('==')
        extra = ''
        tarball_filepath = _create_pkg_tarball(self.tmpdir, pkg_name, version, deps=self.index[pkg_name][version][extra])
        return MockRequestsResponse(url, tarball_filepath)

class MockRequestsResponse:
    def __init__(self, url, filepath):
        'Those are the only attributes/method used by pip/download.py'
        self.url = url
        self.raw = open(filepath, 'rb')
        self.headers= {'content-length': self.raw.tell()}

    def raise_for_status(self):
        pass

def _create_pkg_tarball(tmpdir, pkg_name, version, deps):
    'Creates a tarball containing a single valid setup.py file for the (pkg_name, version, deps) provided'
    tarball_basefilename = pkg_name + '-' + version
    pkg_dir = os.path.join(tmpdir, tarball_basefilename)
    os.mkdir(pkg_dir)
    with open(os.path.join(pkg_dir, 'setup.py'), 'w+') as setup_py:
        setup_py.write('from distutils.core import setup\n')
        setup_py.write('setup(name="' + pkg_name + '",\n')
        setup_py.write('      version="' + version + '",\n')
        setup_py.write('      install_requires=[' + ', '.join('"' + dep + '"' for dep in deps) + '])\n')
    tarball_filepath = os.path.join(tmpdir, tarball_basefilename + '.tar.gz')
    with tarfile.open(tarball_filepath, 'w:gz') as tar:
        tar.add(pkg_dir, arcname=os.path.basename(pkg_dir))
    return tarball_filepath



from nexus_uploader.pypi import get_package_release_from_pypi

# USAGE: PYTHONPATH=. py.test tests/test_get_package_release_from_pypi.py
# AUTHOR: Lucas Cimon


def test_docutils(httpserver):
    with open('tests/fixtures/pypi_docutils_2016-12-13.json', 'r') as mocked_response_content:
        httpserver.serve_content(mocked_response_content.read())
    get_package_release_from_pypi('docutils', '0.13.1', httpserver.url + '/{}/json', allowed_classifiers=('py3-none-any',))



from nexus_uploader.nexus import NexusRepositoryClient

try: # Python 3
    from unittest.mock import patch, Mock
except ImportError: # Python 2
    from mock import patch, Mock
from .pytest_markers import pypi_integration


def test_append_egg_hash_if_nexus_url_ok():
    nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))
    url = 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz'
    assert nexus_client.append_egg_hash_if_nexus_url(url) == 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz#egg=bonjour==1.2.3-SNAPSHOT'

def test_append_egg_hash_if_nexus_url_external():
    nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))
    url = 'http://outside.void/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz'
    assert nexus_client.append_egg_hash_if_nexus_url(url) == url

def test_append_egg_hash_if_nexus_url_hash_already_there():
    nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))
    url = 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz#egg=bonjour==1.2.3-SNAPSHOT'
    assert nexus_client.append_egg_hash_if_nexus_url(url) == url


def test_upload_from_pypi_if_need_be__with_matching_pkgs_on_nexus():
    with patch('nexus_uploader.nexus.requests') as nexus_requests:
        nexus_requests.session.return_value = MockSessionFromFixtures()
        nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))

    nexus_artifact_url = nexus_client.upload_from_pypi_if_need_be('docutils', '0.13.1', 'http://dummy-pypi.wtf/pypi/{}/json', ('py2.py3-none-any', 'py2-none-any'))

    assert nexus_artifact_url == 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/docutils/0.13.1/docutils-0.13.1-py2.py3-none-any.tar.gz'

def test_upload_from_pypi_if_need_be__with_no_matching_pkgs_on_nexus():
    class MockSession404(Mock):
        def get(self, url):
            return MockRequestsResponse(status_code=404)
    mock_session = MockSession404()
    with patch('nexus_uploader.nexus.requests') as nexus_requests:
        nexus_requests.session.return_value = mock_session
        nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))

    with patch('nexus_uploader.pypi.get_package_releases') as pypi_get_package_releases_mock:
        pypi_get_package_releases_mock.return_value = {
            '1.2.3': [
                {
                    'python_version': 'any',
                    'url': 'https://pypi.python.org/packages/5f/6d/e864b3c61b81eec57386ac62082fccfe694c7c3046d8723258a37da6d5fc/mylib-1.2.3-py2-none-any.whl',
                    'filename': 'mylib-1.2.3-py2-none-any.whl'
                },
                {
                    'python_version': 'any',
                    'url': 'https://pypi.python.org/packages/7c/30/8fb30d820c012a6f701a66618ce065b6d61d08ac0a77e47fc7808dbaee47/mylib-1.2.3-py3-none-any.whl',
                    'filename': 'mylib-1.2.3-py3-none-any.whl'
                },
                {
                    'python_version': 'source',
                    'url': 'https://pypi.python.org/packages/05/25/7b5484aca5d46915493f1fd4ecb63c38c333bd32aa9ad6e19da8d08895ae/mylib-1.2.3.tar.gz',
                    'filename': 'mylib-1.2.3.tar.gz'
                }
            ]
        }
        nexus_artifact_url = nexus_client.upload_from_pypi_if_need_be('mylib', '1.2.3', 'http://dummy-pypi.wtf/pypi/{}/json', ('py2.py3-none-any', 'py2-none-any'))

    assert nexus_artifact_url == 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/mylib/1.2.3/mylib-1.2.3-py2.py3-none-any.tar.gz'
    assert mock_session.post.called


class MockSessionFromFixtures(object):
    def get(self, url):
        pkg_name, version = url.split('/')[-2:]
        with open('tests/fixtures/{}-{}.html'.format(pkg_name, version), 'r') as f:
            return MockRequestsResponse(text=f.read())

class MockRequestsResponse:
    def __init__(self, status_code=200, text=''):
        self.content = self.text = text
        self.status_code = status_code
    def raise_for_status(self):
        pass



#
# Copyright (C) 2016 VSCT
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

'Code to parse, write & filter the *requirements.txt files'

import logging, re
from .utils import aslist


logger = logging.getLogger(__name__)


class RequirementError(Exception):
    pass

@aslist
def read_requirements(reqfile_lines):
    for line in reqfile_lines:
        line = line.strip()
        if line and not line.startswith('#'):
            requirement, _, comment = line.partition(' #')
            requirement, comment = requirement.strip(), comment.strip()
            if re.search('^https?://', requirement):
                yield 'url', (requirement, comment)
            elif requirement.startswith('-e '):
                yield 'editable', (requirement[3:], comment.partition(' #')[0].strip())
            elif '==' in requirement:
                pkg_name, pkg_version = requirement.split('==')
                yield 'version-locked', (pkg_name.strip(), pkg_version.strip(), comment)
            else:
                raise RequirementError('Unsupported requirement format: {}'.format(line))

@aslist
def requirements2reqfile_lines(requirements):
    for req_type, req_info in requirements:
        if req_type == 'url':
            url, comment = req_info
            yield url + ' # ' + comment
        elif req_type == 'editable':
            local_path, comment = req_info
            yield '-e ' + local_path + ' # ' + comment
        elif req_type == 'version-locked':
            pkg_name, pkg_version, comment = req_info
            yield pkg_name + '==' + pkg_version + ' # ' + comment

def subst_editable_pkg_fallback(requirement):
    req_type, req_info = requirement
    if req_type != 'editable':
        return requirement
    local_path, comment = req_info
    if comment:
        if comment.startswith('http'):
            return 'url', (comment, '')
        if '==' in comment:
            pkg_name, pkg_version = comment.split('==')
            return 'version-locked', (pkg_name.strip(), pkg_version.strip(), '')
    raise RequirementError('Dependencies in editable mode must have a fallback: {}'.format(local_path))

@aslist
def filter_out_provided_requirements(requirements, provided_requirements):
    provided_requirements = {name: version for req_type, (name, version, comment) in provided_requirements if req_type == 'version-locked'}
    for req_type, req_info in requirements:
        if req_type != 'version-locked':
            yield req_type, req_info
            continue
        pkg_name, pkg_version, comment = req_info
        default_package_version = provided_requirements.get(pkg_name, None)
        if default_package_version:
            if default_package_version == pkg_version:
                logger.debug('Filtering out Python package %s version %s found in provided requirements', pkg_name, pkg_version)
                continue
            logger.warning(('Python package %s exist with version %s (not %s) in provided requirements :'
                            ' if this version match your needs, please use it instead of adding another dependency'),
                           pkg_name, default_package_version, pkg_version)
        yield 'version-locked', (pkg_name, pkg_version, comment)



#
# Copyright (C) 2016 VSCT
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

'Code to interact with Pypi'

import requests

from .utils import aslist


class PypiQueryError(Exception):
    pass

def get_package_release_from_pypi(pkg_name, version, pypi_json_api_url, allowed_classifiers):
    """
    No classifier-based selection of Python packages is currently implemented: for now we don't fetch any .whl or .egg
    Eventually, we should select the best release available, based on the classifier & PEP 425: https://www.python.org/dev/peps/pep-0425/
    E.g. a wheel when available but NOT for tornado 4.3 for example, where available wheels are only for Windows.
    Note also that some packages don't have .whl distributed, e.g. https://bugs.launchpad.net/lxml/+bug/1176147
    """
    matching_releases = get_package_releases_matching_version(pkg_name, version, pypi_json_api_url)
    src_releases = [release for release in matching_releases if release['python_version'] == 'source']
    if src_releases:
        return select_src_release(src_releases, pkg_name, target_classifiers=('py2.py3-none-any',), select_arbitrary_version_if_none_match=True)
    if allowed_classifiers:
        return select_src_release(matching_releases, pkg_name, target_classifiers=allowed_classifiers)
    raise PypiQueryError('No source supported found for package {} version {}'.format(pkg_name, version))

def select_src_release(src_releases, pkg_name, target_classifiers, select_arbitrary_version_if_none_match=False):
    for release in src_releases:
        release['classifier'], release['extension'] = extract_classifier_and_extension(pkg_name, release['filename'])
    try:  # we return the release with the most generic classifier if there is one
        return next(release for release in src_releases if release['classifier'] in target_classifiers)
    except StopIteration:
        if select_arbitrary_version_if_none_match:
            return src_releases[0]

def get_package_releases_matching_version(pkg_name, version, pypi_json_api_url):
    releases = get_package_releases(pkg_name, pypi_json_api_url)
    if version not in releases:
        raise PypiQueryError('Version {} of package {} not found in Pypi. Available versions: {}'.format(version, pkg_name, ','.join(sorted(releases.keys()))))
    return releases[version]

def get_package_releases(pkg_name, pypi_json_api_url):
    response = requests.get(pypi_json_api_url.format(pkg_name))
    response.raise_for_status()
    return response.json()['releases']

def extract_classifier_and_extension(pkg_name, filename):
    """
    Returns a PEP425-compliant classifier (or 'py2.py3-none-any' if it cannot be extracted),
    and the file extension
    TODO: return a classifier 3-members namedtuple instead of a single string
    """
    basename, _, extension = filename.rpartition('.')
    if extension == 'gz' and filename.endswith('.tar.gz'):
        extension = 'tar.gz'
        basename = filename[:-7]
    if basename == pkg_name or basename[len(pkg_name)] != '-':
        return 'py2.py3-none-any', extension
    basename = basename[len(pkg_name)+1:]
    classifier_parts = basename.split('-')
    if len(classifier_parts) < 3:
        return 'py2.py3-none-any', extension
    if len(classifier_parts) == 3:
        _, _, classifier_parts[0] = classifier_parts[0].rpartition('.')
    return '-'.join(classifier_parts[-3:]), extension

@aslist
def filter_allowed_pkg_urls_for_classifiers(pkg_urls, allowed_pkg_classifiers):
    for pkg_url in pkg_urls:
        pkg_name, pkg_filename = pkg_url.split('/')[-3], pkg_url.split('/')[-1]
        classifier, _ = extract_classifier_and_extension(pkg_name, pkg_filename)
        if classifier in allowed_pkg_classifiers:
            yield pkg_url



#
# Copyright (C) 2016 VSCT
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# pylint: disable=deprecated-module

from tempfile import NamedTemporaryFile
try: # Python 3
    from tempfile import TemporaryDirectory
except ImportError: # Python 2
    from contextlib import contextmanager
    import shutil
    from tempfile import mkdtemp
    @contextmanager
    def TemporaryDirectory(): # pylint: disable=invalid-name
        tmp_dir_path = mkdtemp()
        try:
            yield tmp_dir_path
        finally:
            shutil.rmtree(tmp_dir_path)

import optparse, os, pip
from pip.download import unpack_url
from pip.req import parse_requirements
from pip.req.req_install import InstallRequirement
from pip.req.req_set import make_abstract_dist, Installed
from pip.download import PipSession
from piptools.scripts.compile import PipCommand
from piptools.repositories import LocalRequirementsRepository, PyPIRepository
from piptools.resolver import Resolver
from piptools.utils import assert_compatible_pip_version
from piptools.writer import OutputWriter

from .utils import aslist


def pip_compile(reqfile_lines, nexus_hostname, append_egg_hash_to_url_if_need_be):
    assert_compatible_pip_version()
    in_reqfile_lines = []
    for reqfile_line in reqfile_lines:
        if reqfile_line.startswith('http'):
            if ' #' in reqfile_line:
                reqfile_line = reqfile_line.partition(' #')[0]
            reqfile_line = append_egg_hash_to_url_if_need_be(reqfile_line)
        in_reqfile_lines.append(reqfile_line)
    constraints = _constraints_from_reqfile_lines(in_reqfile_lines)
    with TemporaryDirectory() as tmp_dir_path:
        return _pip_compile(constraints, nexus_hostname, append_egg_hash_to_url_if_need_be, tmp_dir_path)

@aslist
def _pip_compile(constraints, nexus_hostname, append_egg_hash_to_url_if_need_be, html_index_dir_path):
    dependency_links_requirements = {str(c.req): c for c in constraints if c.link}

    class CustomResolver(Resolver):
        @staticmethod
        def check_constraints(constraints):
            # Overrides Resolver._check_constraints
            # We allow non-editable URLs as packages
            pass

        def _iter_dependencies(self, ireq):
            # Overrides Resolver._iter_dependencies
            # We use our cached InstallRequirements that have a .link, instead of the new ones built by piptools.resolver.Resolver._iter_dependencies
            for dep_ireq in super(CustomResolver, self)._iter_dependencies(ireq):
                if str(dep_ireq.req) == 'setuptools':
                    continue
                if str(dep_ireq.req) in dependency_links_requirements:
                    yield dependency_links_requirements[str(dep_ireq.req)]
                else:
                    yield dep_ireq

    class CustomPyPIRepository(PyPIRepository):
        def get_dependencies(self, ireq):
            # Overrides PyPIRepository.get_dependencies
            dependencies = set(super(CustomPyPIRepository, self).get_dependencies(ireq))
            dependency_links = self.get_dependency_links(ireq)
            dependency_links = [append_egg_hash_to_url_if_need_be(url) for url in dependency_links]
            dependency_links = [url for url in dependency_links if url]
            dep_links_ireqs = set(InstallRequirement.from_line(url) for url in dependency_links)
            # We cache those dependency_links so that they can be accessed from CustomResolver._iter_dependencies, self.find_all_candidates and at the end of _pip_compile
            for dep_ireq in dep_links_ireqs:
                dep_ireq.remove_temporary_source()
                dependency_links_requirements[str(dep_ireq.req)] = dep_ireq
            return dependencies | dep_links_ireqs

        def get_dependency_links(self, ireq):
            # Reproduce pip.req.req_set.RequirementSet._prepare_file code
            ireq.check_if_exists()
            if ireq.satisfied_by is not None:
                abstract_dist = Installed(ireq)
            else:
                ireq.ensure_has_source_dir(self.source_dir)
                ireq.populate_link(self.finder, upgrade=False, require_hashes=False)
                assert ireq.link
                unpack_url(ireq.link, ireq.source_dir, self._download_dir, only_download=True, session=self.session)
                abstract_dist = make_abstract_dist(ireq)
            abstract_dist.prep_for_dist()
            dist = abstract_dist.dist(self.finder)
            # pylint: disable=protected-access
            return list(dist._get_metadata('dependency_links.txt'))

        def find_all_candidates(self, req_name):
            # Overrides PyPIRepository.find_all_candidates
            # We update our --find-links HTML file
            _write_find_links_html_filename(html_index_dir_path, [dep_ireq.link.url for dep_ireq in dependency_links_requirements.values()])
            return super(CustomPyPIRepository, self).find_all_candidates(req_name)

    # Reproduce piptools.scripts.compile.cli code
    pip_options, session = _get_pip_options(['--trusted-host', nexus_hostname, '--find-links', html_index_dir_path])
    repository = LocalRequirementsRepository(existing_pins=dict(), proxied_repository=CustomPyPIRepository(pip_options, session))
    resolver = CustomResolver(constraints, repository, clear_caches=True)
    results = resolver.resolve()

    out_reqfile_lines = _capture_annotated_out_reqlines(results, resolver=resolver, constraints=constraints, format_control=repository.finder.format_control)

    for requirement in out_reqfile_lines:
        eol_comment = ''
        if ' #' in requirement and not requirement.startswith('#'):
            requirement, eol_comment = requirement.split(' #')
        requirement, eol_comment = requirement.strip(), eol_comment.strip()
        if requirement in dependency_links_requirements:
            yield dependency_links_requirements[requirement].link.url + (' # ' + eol_comment if eol_comment else '')
        else:
            yield requirement + (' # ' + eol_comment if eol_comment else '')

def _constraints_from_reqfile_lines(reqfile_lines):
    with NamedTemporaryFile('w') as tmp_file:
        tmp_file.write('\n'.join(reqfile_lines))
        tmp_file.flush()
        return list(parse_requirements(tmp_file.name, session=PipSession()))

def _get_pip_options(pip_args):
    # Reproduce piptools.scripts.compile.cli code
    pip_command = PipCommand()
    pip.cmdoptions.make_option_group(pip.cmdoptions.index_group, pip_command.parser)
    pip_command.parser.add_option(optparse.Option('--pre', action='store_true', default=False))
    pip_options, _ = pip_command.parse_args(pip_args)
    # pylint: disable=protected-access
    session = pip_command._build_session(pip_options)
    return pip_options, session

def _write_find_links_html_filename(html_index_dir_path, find_links_urls):
    html_content = '\n'.join("<a href='{}'></a>".format(url) for url in find_links_urls)
    with open(os.path.join(html_index_dir_path, 'index.html'), 'w') as html_index_file:
        html_index_file.write(html_content)

def _capture_annotated_out_reqlines(results, resolver, constraints, format_control):
    with NamedTemporaryFile(delete=False) as tmp_file:
        tmp_file_name = tmp_file.name
    try:
        writer = CustomOutputWriter(src_files=None, dst_file=tmp_file_name, dry_run=False,
                              emit_header=False, emit_index=False, emit_trusted_host=False,
                              annotate=True, generate_hashes=False, default_index_url=None, index_urls=None,
                              trusted_hosts=(), format_control=format_control)
        writer.write(results=results, reverse_dependencies=resolver.reverse_dependencies(results),
                     primary_packages={ireq.req.key for ireq in constraints},
                     markers={}, hashes=None)
        with open(tmp_file_name, 'r') as tmp_file:
            return tmp_file.readlines()
    finally:
        os.unlink(tmp_file_name)



# This is a workaround for pip 8.1.1 and lower while the fix isn't merged: https://github.com/jazzband/pip-tools/pull/540
# pylint: disable=too-many-arguments,wrong-import-order,wrong-import-position
from piptools.utils import comment, key_from_req, UNSAFE_PACKAGES

class CustomOutputWriter(OutputWriter):
    def _iter_lines(self, results, reverse_dependencies, primary_packages, markers, hashes):
        for line in self.write_header():
            yield line
        for line in self.write_flags():
            yield line

        unsafe_packages = {r for r in results if r.name in UNSAFE_PACKAGES}
        packages = {r for r in results if r.name not in UNSAFE_PACKAGES}

        packages = sorted(packages, key=self._sort_key)
        unsafe_packages = sorted(unsafe_packages, key=self._sort_key)

        for ireq in packages:
            line = self._format_requirement(
                ireq, reverse_dependencies, primary_packages,
                markers.get(key_from_req(ireq.req)), hashes=hashes)
            yield line

        if unsafe_packages:
            yield ''
            yield comment('# The following packages are considered to be unsafe in a requirements file:')

            for ireq in unsafe_packages:

                yield self._format_requirement(ireq,
                                               reverse_dependencies,
                                               primary_packages,
                                               marker=markers.get(ireq.req.name),
                                               hashes=hashes)



import random, requests, sys
from nexus_uploader.pypi import extract_classifier_and_extension, get_package_releases

# USAGE: PYTHONPATH=. ipython3 --pdb tests/smoke_test_extract_classifier_and_extension.py 100
# AUTHOR: Lucas Cimon

def main(random_releases_count):
    print('# Extracting classifier & extension from {} packages randomly picked on Pypi (no output means OK)'.format(random_releases_count))
    for pkg_name, _, pkg_release in random_packages_releases(random_releases_count):
        print(pkg_name)
        classifier, extension = extract_classifier_and_extension(pkg_name, pkg_release['filename'])
        print('{0:<70} -> {1:<30} {2}'.format(pkg_release['filename'], classifier, extension))

def random_packages_releases(random_releases_count):
    all_pypi_packages = get_all_pypi_packages()
    i = 0
    while i < random_releases_count:
        pkg_name = random.choice(all_pypi_packages)
        try:
            pkg_releases = get_package_releases(pkg_name, 'http://pypi.python.org/pypi/{}/json')
        except requests.HTTPError:
            print('!!! WARNING: No release found for package name ', pkg_name)
            continue
        version = random.choice(list(pkg_releases.keys()))
        version_pkg_releases = pkg_releases[version]
        if version_pkg_releases:
            i += 1
            yield pkg_name, version, random.choice(version_pkg_releases)

def get_all_pypi_packages():
    """
    HTML structure at /simple/ :
        <html><head><title>Simple Index</title><meta name="api-version" value="2" /></head><body>
        <a href='199fix'>199Fix</a><br/>
        ...
        <a href='zzzfs'>zzzfs</a><br/>
        </body></html>
    """
    response = requests.get('https://pypi.python.org/simple/')
    response.raise_for_status()
    all_lines = [line.decode('ascii') for line in response.iter_lines()][1:-1]
    return [line[9:line.index("'", 9)] for line in all_lines]

if __name__ == '__main__':
    main(int(sys.argv[1]))



from nexus_uploader.pip_compile import pip_compile
from nexus_uploader.requirements import read_requirements

try: # Python 3
    from unittest.mock import patch
except ImportError: # Python 2
    from mock import patch
from .pytest_markers import pypi_integration


@pypi_integration  # only ran with --run-pypi-integration-tests
def test_pipcompile_with_real_repository():
    reqs_gen = pip_compile(['nexus_uploader'],
                           nexus_hostname='dummy-nexus.wtf',
                           append_egg_hash_to_url_if_need_be=raise_if_called)
    assert len(read_requirements(reqs_gen)) == 6


def test_pipcompile_with_mocked_repository(FakePypiRepository):
    with patch('nexus_uploader.pip_compile.PyPIRepository', FakePypiRepository):
        reqs_gen = pip_compile(['lib1'],
                               nexus_hostname='dummy-nexus.wtf',
                               append_egg_hash_to_url_if_need_be=raise_if_called)
        assert read_requirements(reqs_gen) == \
            [('version-locked', ('lib1', '1.2.3', '')), ('version-locked', ('lib2', '4.5.7', 'via lib1'))]


def raise_if_called(*args, **kwargs):
    raise NotImplementedError



#!/usr/bin/env python

import io, sys
from os import path
from setuptools import setup, find_packages

version = '1.0.10'
setup_requires = []

if 'nexus_upload' in sys.argv:
    sys.argv.remove('nexus_upload')
    setup_requires.append('nexus_uploader')

if '--snapshot' in sys.argv:
    sys.argv.remove('--snapshot')
    version += '-SNAPSHOT'

here = path.abspath(path.dirname(__file__))

# Using .rst as long as Markdown is not properly supported by pypi/warehouse :( -> https://github.com/pypa/warehouse/issues/869
with io.open(path.join(here, 'README.rst'), encoding='utf-8') as readme_file:
    readme = readme_file.read()

with io.open(path.join(here, 'CHANGELOG.md'), encoding='utf-8') as changelog_file:
    changelog = changelog_file.read()

with io.open(path.join(here, 'requirements.txt')) as requirements_file:
    requirements = requirements_file.readlines()

setup(
    name='nexus_uploader',
    description='CLI tool to upload Python packages listed in a requirements.txt file into a Sonatype Nexus (from Pypi), and return the list of the artifact URLs',
    long_description=readme + '\n\n' + changelog,
    author='Lucas Cimon',
    url='http://github.com/voyages-sncf-technologies/nexus_uploader',
    install_requires=requirements,
    packages=find_packages(),
    version=version,
    setup_requires=setup_requires,
    zip_safe=True,  # http://peak.telecommunity.com/DevCenter/setuptools#setting-the-zip-safe-flag
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2',
        'Operating System :: OS Independent',
        'Topic :: Utilities',
    ],
    keywords='nexus artifacts packages upload requirements pip-compile',
    license='Apache License 2.0',
    include_package_data=True,  # Active la prise en compte du fichier MANIFEST.in
    entry_points={
        'console_scripts': ['pyRequirements2nexus = nexus_uploader.pyrequirements2nexus:main'],
        'distutils.commands': ['nexus_upload = nexus_uploader.nexus_upload_cmd:NexusUploadCmd'],
    },
)



from nexus_uploader.pypi import extract_classifier_and_extension
try:
    import pytest
    xfail = pytest.mark.xfail
except ImportError:
    xfail = lambda f: f  # dummy decorator

# USAGE: PYTHONPATH=. py.test tests/test_extract_classifier_and_extension.py
# AUTHOR: Lucas Cimon

def test_no_classifier():
    classifier, extension = extract_classifier_and_extension('luigi', 'luigi-1.1.1.tar.gz')
    assert extension == 'tar.gz'
    assert classifier == 'py2.py3-none-any'

def test_osx_targz():
    classifier, extension = extract_classifier_and_extension('luigi', 'luigi-1.0.9.macosx-10.7-intel.tar.gz')
    assert extension == 'tar.gz'
    assert classifier == 'macosx-10.7-intel'

def test_windows_wheel():
    classifier, extension = extract_classifier_and_extension('tornado', 'tornado-4.3b2-cp35-none-win32.whl')
    assert extension == 'whl'
    assert classifier == 'cp35-none-win32'

def test_maven_snapshot_package():
    classifier, extension = extract_classifier_and_extension('logdorak', 'logdorak-1.0-SNAPSHOT-py2.py3-none-any.tar.gz')
    assert extension == 'tar.gz'
    assert classifier == 'py2.py3-none-any'

@xfail
def test_incorrect_package_name1():
    classifier, extension = extract_classifier_and_extension('asv_media', 'asv_media-dev-20121101-01.tar.gz')
    assert extension == 'tar.gz'
    assert classifier == 'py2.py3-none-any'

@xfail
def test_incorrect_package_name2():
    classifier, extension = extract_classifier_and_extension('freetype', 'Freetype-Milestone1-py2.4-win32.egg')
    assert extension == 'tar.gz'
    assert classifier == 'py2.4-none-win32'

@xfail
def test_incorrect_platform_detection():
    classifier, extension = extract_classifier_and_extension('rawphoto', 'rawphoto-0.3.0.linux-x86_64.tar.gz')
    assert extension == 'tar.gz'
    assert classifier == 'py2.py3-none-linux-x86_64'
    classifier, extension = extract_classifier_and_extension('maltfy', 'maltfy-v0.3.2.linux-i686.tar.gz')
    assert extension == 'tar.gz'
    assert classifier == 'py2.py3-none-linux-i686'
    classifier, extension = extract_classifier_and_extension('mathtools', 'mathtools-1.2.win-amd64.exe')
    assert extension == 'tar.gz'
    assert classifier == 'py2.py3-none-win-amd64'

@xfail
def test_incorrect_python_tag_detection():
    classifier, extension = extract_classifier_and_extension('instatrace', 'instatrace-1.0.0-py2.6.egg')
    assert extension == 'tar.gz'
    assert classifier == 'py2.6-none-any'
    classifier, extension = extract_classifier_and_extension('zc.recipe.zope3checkout', 'zc.recipe.zope3checkout-1.2-py2.4.egg')
    assert extension == 'tar.gz'
    assert classifier == 'py2.4-none-any'
